#ifndef NSAPP_H
#define NSAPP_H

#include <pthread.h>
#include "map.h"
#include "alg.h"

#define MAX_LEN 32
#define MAX_HASH 8

typedef struct board {
	int id;
	char name[9];
	pthread_t thread;
	struct wu *wu[15];
	int ready[15];
	struct hwctx *hw;
	unsigned irq_mask;
	unsigned irq_buf[8];
	int irq_ptr;
	int percent, len, pfx_len;
	map_t *map;
	alg_hash *alg[15];
} board_t;

typedef struct wu {
	char prefix[MAX_LEN];
	board_t *board;
	int chip;
	struct task *task;
	struct wu *next;
} wu_t;

typedef struct task {
	int len, pfx_len, wu_size;
	char prefix[MAX_LEN];
	int hashlen, nhash;
	unsigned *hashes;
	map_t *map;
	alg_hash *alg;
} task_t;

#define MSG_DBG 0
#define MSG_INFO 1
#define MSG_WARN 2
#define MSG_FAIL 3
void message(int, char *, ...);
#define return_message(severity, ...) do { message(severity, __VA_ARGS__); return; } while(0)
#define retval_message(retval, severity, ...) do { message(severity, __VA_ARGS__); return retval; } while(0)

extern unsigned char hamilton[64];

extern board_t *boards;
extern int nboards;

extern pthread_mutex_t wu_mutex;
extern wu_t *wu_todo;
extern int wu_todo_cnt, wu_todo_max;
extern wu_t *wu_done;

int wu_lock(void);
wu_t *wu_pop_todo(void);	// lock & pop
void wu_push_todo(wu_t *);	// push & unlock
int wu_push_done(wu_t *);	// lock, push, unlock, return % done
int wu_unlock(void);

#endif
